/* kazio.js
 * by Fabien Cazenave (Kaze)
 *
 * File IO library (mostly inspired by Launchy), shared with NsmConText.
 * Works on most platforms, but still can't pass arguments on Mac OS X
 */

var NsmFileIO = {
	
	urlToPath : function(data) {
		// taken from Launchy (io.js)
		const hasUnescape = (typeof(unescape)=="function");
		var path = null;
		
		try {
			var uri = Components.classes['@mozilla.org/network/standard-url;1']
				.createInstance(Components.interfaces.nsIURL);
			var file = Components.classes['@mozilla.org/file/local;1']
				.createInstance(Components.interfaces.nsILocalFile);
			uri.spec = data;
				
			try { // normal OS
				file.initWithPath(uri.path);
				path = hasUnescape ? unescape(file.path) : file.path;
			} catch (e) {}
			try { // Widows sucks
				file.initWithPath(uri.path.replace(/^\//,"").replace(/\//g,"\\"));
				path = hasUnescape ? unescape(file.path) : file.path;
			} catch (e) {}
		} catch(e) {
			NsmTrace("could not get path from " + data);
		}
		
		return path;
	},

	write : function(filePath, data) {
		var file = this._newLocalFile(filePath);
		
		if ( file.exists() == false ) {
			//~ alert( "Creating file... " );
			file.create( Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 420 );
		}
		var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
			.createInstance( Components.interfaces.nsIFileOutputStream );
			
		/* Open flags 
		#define PR_RDONLY       0x01
		#define PR_WRONLY       0x02
		#define PR_RDWR         0x04
		#define PR_CREATE_FILE  0x08
		#define PR_APPEND      0x10
		#define PR_TRUNCATE     0x20
		#define PR_SYNC         0x40
		#define PR_EXCL         0x80
		*/
		/*
		** File modes ....
		**
		** CAVEAT: 'mode' is currently only applicable on UNIX platforms.
		** The 'mode' argument may be ignored by PR_Open on other platforms.
		**
		**   00400   Read by owner.
		**   00200   Write by owner.
		**   00100   Execute (search if a directory) by owner.
		**   00040   Read by group.
		**   00020   Write by group.
		**   00010   Execute by group.
		**   00004   Read by others.
		**   00002   Write by others
		**   00001   Execute by others.
		**
		*/
		outputStream.init( file, 0x04 | 0x08 | 0x20, 420, 0 );
		var result = outputStream.write(data, data.length );
		outputStream.close();
		
		return result;
	},
	
	read : function(filePath)  {
		var file = this._newLocalFile(filePath);
		
		if ( file.exists() == false )
			return false;
			//~ alert(filePath + " does not exist");

		var is = Components.classes["@mozilla.org/network/file-input-stream;1"]
			.createInstance( Components.interfaces.nsIFileInputStream );
		is.init( file,0x01, 00004, null);
		
		var sis = Components.classes["@mozilla.org/scriptableinputstream;1"]
			.createInstance( Components.interfaces.nsIScriptableInputStream );
		sis.init( is );
		
		return sis.read( sis.available() );
	},
	
	unlink : function(filePath) {
		var file = this._newLocalFile(filePath);
		try {
			file.remove(false);
			return true;
		} catch(e) {
			//~ alert(e);
			return false;
		}
	},
	
	_newLocalFile : function(filePath) {
		//~ try {
			//~ netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
		//~ } catch (e) {
			//~ alert("Permission to save file was denied.");
		//~ }
		try {
			var nsFile = Components.classes['@mozilla.org/file/local;1']
				.createInstance(Components.interfaces.nsILocalFile);
			nsFile.initWithPath(filePath);
		} catch(e) {
			NsmTrace("could not create LocalFile interface on " + FilePath);
		}
		
//~ var msg = "This file is getting edited.";
//~ try {
	//~ msg = "This file is getting edited."
		//~ + "\n path = " + nsFile.path
		//~ + "\n" + nsFile.fileSize
		//~ + "\n" + nsFile.lastModifiedTime;
//~ } catch (e) {
//~ }
//~ AlertWithTitle("HandCoder", msg);
		
		return nsFile;
	},
	
	_newProcess : function(appFile) {
		// more info about processes: http://www.xulplanet.com/references/xpcomref/comps/c_processutil1.html
		var process = null;
		
		//~ if (appFile.isFile() && appFile.isExecutable()) try { // on MacOS X, apps are folders (bug in launchy 4.0.0, apps.js, line 473)
		if (appFile.isExecutable()) try {
			process = Components.classes['@mozilla.org/process/util;1']
				.createInstance(Components.interfaces.nsIProcess);
			process.init(appFile);
		} catch(e) {
			NsmTrace("failed to create process.");
		}
		
		return process;
	},
	
	_newMIMEInfo : function(dataFile) {
		var mimeInfo = null;
		
		//~ if (appFile.isFile() && appFile.isExecutable()) try { // on MacOS X, apps are folders (bug in launchy 4.0.0, apps.js, line 473)
		//~ if (appFile.isExecutable()) try {
		try {
			var mimeSvc = Components.classes['@mozilla.org/uriloader/external-helper-app-service;1']
				.getService(Components.interfaces.nsIMIMEService);
			mimeInfo = mimeSvc.getFromTypeAndExtension(dataFile, null);
		} catch(e) {
			NsmTrace("failed to create MIME service.");
		}
		
		return mimeInfo;
	},
	
	run : function(filePath, dataPath, args, blocking) {
		var result = 0;
		var appFile = this._newLocalFile(filePath);
			
		if (args) { // will NOT work on MacOS X, sorry :-(
			args.push(dataPath);
			var len = args.length;
			try {
				var process = this._newProcess(appFile);
				result = process.run(blocking, args, len);
//~ NsmExternalEditorProcess = process;
//~ process.kill();
//~ process.abort();
//~ process.stop();
				NsmTrace("started application '" + filePath + "' with " + len + " params: '" + args + "'");
			} catch(e) {
				NsmTrace("failed to run application '" + filePath + "' with " + len + " params: '" + args + "'");
			}
		} else { // MacOS X hack: use nsIMIMEService if there is no other arguments than the file to edit
			try { // taken from ViewSourceWith :-/
				var dataFile = this._newLocalFile(dataPath);
				var mimeInfo = this._newMIMEInfo(dataFile);
				mimeInfo.alwaysAskBeforeHandling = false;
				mimeInfo.preferredAction = Components.interfaces.nsIMIMEInfo.useHelperApp;
				mimeInfo.preferredApplicationHandler = appFile;
				mimeInfo.launchWithFile(dataFile);
				
				NsmTrace("started application '" + filePath + "' on '" + dataPath + "'");
			} catch(e) {
				NsmTrace("failed to run application '" + filePath + "' on '" + dataPath + "'");
			}
		}
		
 		return result;
	}
}
function NsmTrace(message) {
	var CONSOLE_SERVICE = Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService);
	try {
		CONSOLE_SERVICE.logStringMessage("HandCoder: " + message);
	} catch(ex) {}
}

/* File IO : flags & modes
	var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
		.createInstance( Components.interfaces.nsIFileOutputStream );
	outputStream.init( file, 0x04 | 0x08 | 0x20, 420, 0 );
*/		
	/* Open flags 
	#define PR_RDONLY       0x01
	#define PR_WRONLY       0x02
	#define PR_RDWR         0x04
	#define PR_CREATE_FILE  0x08
	#define PR_APPEND      0x10
	#define PR_TRUNCATE     0x20
	#define PR_SYNC         0x40
	#define PR_EXCL         0x80
	*/
	/* File modes ....
	**
	** CAVEAT: 'mode' is currently only applicable on UNIX platforms.
	** The 'mode' argument may be ignored by PR_Open on other platforms.
	**
	**   00400   Read by owner.
	**   00200   Write by owner.
	**   00100   Execute (search if a directory) by owner.
	**   00040   Read by group.
	**   00020   Write by group.
	**   00010   Execute by group.
	**   00004   Read by others.
	**   00002   Write by others
	**   00001   Execute by others.
	**
	*/
